#!/usr/bin/python3
from PIL import Image
from itertools import chain
import argparse
import sys


def encode(bitmap, data):
    bitmap = bytearray(bitmap)

    data = chain(
        *map(lambda x: bin(x).replace('0b', '').rjust(8, '0'), data)
    )
    for idx, bit in enumerate(data): 
        if idx >= len(bitmap):
            break
        bit = int(bit)
        bitmap[idx] &= ~1
        bitmap[idx] |= bit

    return bytes(bitmap)

def decode(bitmap):
    res = bytearray()
    bits = ''
    for i in bitmap:
        bits += str(i&1)
        if len(bits) == 8:
            res.append(int(bits, 2))
            bits = ''
    return res

def main_encode(img_path, data_path, out_path):
    img = Image.open(img_path)
    with open(data_path, 'rb') as f:
        data = f.read()
    bitmap = img.tobytes()
    bitmap = encode(bitmap, data)

    result = Image.frombytes(img.mode, img.size, bitmap)
    result.save(out_path)

def main_decode(img_path, out_path):
    img = Image.open(img_path)
    bitmap = img.tobytes()
    bitmap = decode(bitmap)
    if out_path != 'stdout':
        with open(out_path, 'wb') as f:
            f.write(bitmap)
    else:
        sys.stdout.write(bitmap)


def main():
    parser = argparse.ArgumentParser()
    opt_group = parser.add_mutually_exclusive_group()
    opt_group.add_argument(
        "-d", "--decode", action="store_true", help="decode data from image")
    opt_group.add_argument(
        "-e", "--encode", action="store_true", help="encode data into image")
    parser.add_argument(
        '-dt', '--data', type=str, help='path to data you want to encode')
    parser.add_argument('input', type=str, help='path to the input file')
    parser.add_argument(
        '-o',
        '--output',
        type=str,
        help='path to the output file',
        default='stdout')
    args = parser.parse_args()

    if args.decode:
        main_decode(args.input, args.output)
    elif args.encode:
        main_encode(args.input, args.data, args.output)


if __name__ == '__main__':
    main()